package ovh.herisson.Clyde.Services;

import org.springframework.stereotype.Service;
import ovh.herisson.Clyde.Tables.InscriptionRequest;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.Token;
import ovh.herisson.Clyde.Tables.User;

import java.util.Date;
import java.util.HashMap;

@Service
public class AuthenticatorService {

    private final TokenService tokenService;
    private final UserService userService;
    private final InscriptionService inscriptionService;

    public AuthenticatorService(TokenService tokenService, UserService userService, InscriptionService inscriptionService){
        this.tokenService = tokenService;
        this.userService = userService;
        this.inscriptionService = inscriptionService;
    }

    public User getUserFromToken(String token){
        return tokenService.getUserFromToken(token);
    }


    public String login(String identifier, String password, Date expirationDate){
        User user = userService.getUser(identifier);
        if (user == null){return null;}
        if (!userService.checkPassword(user,password)){return null;}
        String token = tokenService.generateNewToken();
        tokenService.saveToken(new Token(user, token,expirationDate));
        return token;
    }

    public InscriptionRequest register(InscriptionRequest inscriptionRequest) {
        return inscriptionService.save(inscriptionRequest);
    }

    public boolean isNotIn(Role[] roles, String token){
        if (token == null)
            return true;

        User poster = getUserFromToken(token);
        if (poster == null) return true;

        for (Role r:roles){
            if (poster.getRole() == r)
                return false;
        }
        return true;
    }



    /** return user's data except password
     * @param user the user to return
     * @return all the user data without the password
     */
    public HashMap<String,Object> userWithoutPassword(User user){
        HashMap<String,Object> toReturn = new HashMap<>();
        toReturn.put("regNo",user.getRegNo());
        toReturn.put("lastName",user.getLastName());
        toReturn.put("firstName",user.getFirstName());
        toReturn.put("email", user.getEmail());
        toReturn.put("address",user.getAddress());
        toReturn.put("birthDate",user.getBirthDate());
        toReturn.put("country",user.getCountry());
        toReturn.put("profilePictureUrl",user.getProfilePictureUrl());
        toReturn.put("role",user.getRole());
        return toReturn;
    }
}

